@@ -22,8 +22,7 @@
 #'   non-finite values are `NA`, `Inf`, `-Inf` and `NaN`. Setting this
 #'   to something other than `NULL` can be expensive because the whole
 #'   vector needs to be traversed and checked.
-#' @param encoding Expected encoding of a string or character
-#'   vector. One of `UTF-8`, `latin1`, or `unknown`.
+#' @param encoding Defunct as of rlang 0.4.0.
 #' @seealso [bare-type-predicates] [scalar-type-predicates]
 #' @name type-predicates
 NULL
@@ -66,14 +65,14 @@ is_double <- function(x, n = NULL, finite = NULL) {
 #' @rdname type-predicates
 is_character <- function(x, n = NULL, encoding = NULL) {
   if (!is_null(encoding)) {
-    warn_deprecated("The `encoding` argument is deprecated as of rlang 0.3.0.")
+    stop_defunct("The `encoding` argument is deprecated as of rlang 0.3.0.")
   }
   if (!.Call(rlang_is_character, x, n)) {
     return(FALSE)
   }
   if (!is_null(encoding)) {
     stopifnot(typeof(encoding) == "character")
-    if (!all(chr_encoding(x) %in% encoding)) {
+    if (!all(Encoding(x) %in% encoding)) {
       return(FALSE)
     }
   }
@@ -101,8 +100,16 @@ is_null <- function(x) {
 
 #' Scalar type predicates
 #'
+#' @description
+#'
 #' These predicates check for a given type and whether the vector is
 #' "scalar", that is, of length 1.
+#'
+#' In addition to the length check, `is_string()` and `is_bool()`
+#' return `FALSE` if their input is missing. This is useful for
+#' type-checking arguments, when your function expects a single string
+#' or a single `TRUE` or `FALSE`.
+#'
 #' @inheritParams type-predicates
 #' @param x object to be tested.
 #' @seealso [type-predicates], [bare-type-predicates]
@@ -138,7 +145,7 @@ is_scalar_double <- function(x) {
 #' @rdname scalar-type-predicates
 is_scalar_character <- function(x, encoding = NULL) {
   if (!is_null(encoding)) {
-    warn_deprecated("The `encoding` argument is deprecated as of rlang 0.3.0.")
+    stop_defunct("The `encoding` argument is deprecated as of rlang 0.3.0.")
   }
   is_character(x, encoding = encoding, n = 1L)
 }
@@ -153,17 +160,17 @@ is_scalar_raw <- function(x) {
   .Call(rlang_is_raw, x, 1L)
 }
 #' @export
+#' @param string A string to compare to `x`. If a character vector,
+#'   returns `TRUE` if at least one element is equal to `x`.
 #' @rdname scalar-type-predicates
-is_string <- function(x, encoding = NULL) {
-  if (!is_null(encoding)) {
-    warn_deprecated("The `encoding` argument is deprecated as of rlang 0.3.0.")
-  }
-  is_character(x, encoding = encoding, n = 1L) && !is.na(x)
+is_string <- function(x, string = NULL) {
+  .Call(rlang_is_string, x, string)
 }
 #' @export
 #' @rdname scalar-type-predicates
 is_scalar_bytes <- is_scalar_raw
-
+#' @export
+#' @rdname scalar-type-predicates
 is_bool <- function(x) {
   is_logical(x, n = 1) && !is.na(x)
 }
@@ -222,7 +229,7 @@ is_bare_numeric <- function(x, n = NULL) {
 #' @rdname bare-type-predicates
 is_bare_character <- function(x, n = NULL, encoding = NULL) {
   if (!is_null(encoding)) {
-    warn_deprecated("The `encoding` argument is deprecated as of rlang 0.3.0.")
+    stop_defunct("The `encoding` argument is deprecated as of rlang 0.3.0.")
   }
   !is.object(x) && is_character(x, n, encoding = encoding)
 }
@@ -337,55 +344,7 @@ is_scalar_integerish <- function(x, finite = NULL) {
   .Call(rlang_is_integerish, x, 1L, finite)
 }
 
-#' Base type of an object
-#'
-#' @description
-#'
-#' \Sexpr[results=rd, stage=render]{rlang:::lifecycle("experimental")}
-#' \Sexpr[results=rd, stage=render]{rlang:::lifecycle("questioning")}
-#'
-#' This is equivalent to [base::typeof()] with a few differences that
-#' make dispatching easier:
-#' * The type of one-sided formulas is "quote".
-#' * The type of character vectors of length 1 is "string".
-#' * The type of special and builtin functions is "primitive".
-#'
-#'
-#' @section Life cycle:
-#'
-#' `type_of()` is an experimental function. Expect API changes.
-#'
-#' @param x An R object.
-#' @export
-#' @keywords internal
-#' @examples
-#' type_of(10L)
-#'
-#' # Quosures are treated as a new base type but not formulas:
-#' type_of(quo(10L))
-#' type_of(~10L)
-#'
-#' # Compare to base::typeof():
-#' typeof(quo(10L))
-#'
-#' # Strings are treated as a new base type:
-#' type_of(letters)
-#' type_of(letters[[1]])
-#'
-#' # This is a bit inconsistent with the core language tenet that data
-#' # types are vectors. However, treating strings as a different
-#' # scalar type is quite helpful for switching on function inputs
-#' # since so many arguments expect strings:
-#' switch_type("foo", character = abort("vector!"), string = "result")
-#'
-#' # Special and builtin primitives are both treated as primitives.
-#' # That's because it is often irrelevant which type of primitive an
-#' # input is:
-#' typeof(list)
-#' typeof(`$`)
-#' type_of(list)
-#' type_of(`$`)
-type_of <- function(x) {
+type_of_ <- function(x) {
   type <- typeof(x)
   if (is_formulaish(x)) {
     if (identical(node_car(x), colon_equals_sym)) {
@@ -402,119 +361,13 @@ type_of <- function(x) {
   }
 }
 
-#' Dispatch on base types
-#'
-#' @description
-#'
-#' \Sexpr[results=rd, stage=render]{rlang:::lifecycle("experimental")}
-#' \Sexpr[results=rd, stage=render]{rlang:::lifecycle("questioning")}
-#'
-#' `switch_type()` is equivalent to
-#' \code{\link[base]{switch}(\link{type_of}(x, ...))}, while
-#' `switch_class()` switchpatches based on `class(x)`. The `coerce_`
-#' versions are intended for type conversion and provide a standard
-#' error message when conversion fails.
-#'
-#'
-#' @section Life cycle:
-#'
-#' * Like [type_of()], `switch_type()` and `coerce_type()` are
-#'   experimental functions.
-#'
-#' * `switch_class()` and `coerce_class()` are experimental functions.
-#'
-#' @param .x An object from which to dispatch.
-#' @param ... Named clauses. The names should be types as returned by
-#'   [type_of()].
-#' @param .to This is useful when you switchpatch within a coercing
-#'   function. If supplied, this should be a string indicating the
-#'   target type. A catch-all clause is then added to signal an error
-#'   stating the conversion failure. This type is prettified unless
-#'   `.to` inherits from the S3 class `"AsIs"` (see [base::I()]).
-#' @seealso [switch_lang()]
-#' @export
-#' @keywords internal
-#' @examples
-#' switch_type(3L,
-#'   double = "foo",
-#'   integer = "bar",
-#'   "default"
-#' )
-#'
-#' # Use the coerce_ version to get standardised error handling when no
-#' # type matches:
-#' to_chr <- function(x) {
-#'   coerce_type(x, "a chr",
-#'     integer = as.character(x),
-#'     double = as.character(x)
-#'   )
-#' }
-#' to_chr(3L)
-#'
-#' # Strings have their own type:
-#' switch_type("str",
-#'   character = "foo",
-#'   string = "bar",
-#'   "default"
-#' )
-#'
-#' # Use a fallthrough clause if you need to dispatch on all character
-#' # vectors, including strings:
-#' switch_type("str",
-#'   string = ,
-#'   character = "foo",
-#'   "default"
-#' )
-#'
-#' # special and builtin functions are treated as primitive, since
-#' # there is usually no reason to treat them differently:
-#' switch_type(base::list,
-#'   primitive = "foo",
-#'   "default"
-#' )
-#' switch_type(base::`$`,
-#'   primitive = "foo",
-#'   "default"
-#' )
-#'
-#' # closures are not primitives:
-#' switch_type(rlang::switch_type,
-#'   primitive = "foo",
-#'   "default"
-#' )
-switch_type <- function(.x, ...) {
-  switch(type_of(.x), ...)
-}
-#' @rdname switch_type
-#' @export
-coerce_type <- function(.x, .to, ...) {
-  switch(type_of(.x), ..., abort_coercion(.x, .to))
-}
-#' @rdname switch_type
-#' @export
-switch_class <- function(.x, ...) {
-  switch(class(.x), ...)
-}
-#' @rdname switch_type
-#' @export
-coerce_class <- function(.x, .to, ...) {
-  switch(class(.x), ..., abort_coercion(.x, .to))
-}
-abort_coercion <- function(x, to_type) {
-  x_type <- friendly_type_of(x)
-  if (!inherits(to_type, "AsIs")) {
-    to_type <- as_friendly_type(to_type)
-  }
-  abort(paste0("Can't convert ", x_type, " to ", to_type))
-}
-
 #' Format a type for error messages
 #'
 #' @section Life cycle:
 #'
-#' * Like [type_of()], `friendly_type()` is experimental.
+#' * `friendly_type()` is experimental.
 #'
-#' @param type A type as returned by [type_of()] or [lang_type_of()].
+#' @param type A type as returned by [typeof()].
 #' @return A string of the prettified type, qualified with an
 #'   indefinite article.
 #' @export
@@ -525,22 +378,7 @@ abort_coercion <- function(x, to_type) {
 #' friendly_type("string")
 #' @export
 friendly_type <- function(type) {
-  friendly <- as_friendly_type(type)
-  if (!is_null(friendly)) {
-    return(friendly)
-  }
-
-  friendly <- friendly_lang_type_of(type)
-  if (!is_null(friendly)) {
-    return(friendly)
-  }
-
-  friendly <- friendly_expr_type_of(type)
-  if (!is_null(friendly)) {
-    return(friendly)
-  }
-
-  type
+  as_friendly_type(type) %||% type
 }
 
 friendly_type_of <- function(x, length = FALSE) {
@@ -600,138 +438,6 @@ paste_classes <- function(x) {
   paste(class(x), collapse = "/")
 }
 
-friendly_lang_type_of <- function(type) {
-  switch(type,
-    named = "a named call",
-    namespaced = "a namespaced call",
-    recursive = "a recursive call",
-    inlined = "an inlined call"
-  )
-}
-
-friendly_expr_type_of <- function(type) {
-  switch(type,
-    NULL = "NULL",
-    name = ,
-    symbol = "a symbol",
-    language = "a call",
-    pairlist = "a pairlist node",
-    literal = "a syntactic literal",
-    missing = "the missing argument"
-  )
-}
-
-#' Dispatch on call type
-#'
-#' @description
-#'
-#' \Sexpr[results=rd, stage=render]{rlang:::lifecycle("experimental")}
-#' \Sexpr[results=rd, stage=render]{rlang:::lifecycle("questioning")}
-#'
-#' `switch_lang()` dispatches clauses based on the subtype of call, as
-#' determined by `lang_type_of()`. The subtypes are based on the type
-#' of call head (see details).
-#'
-#' @details
-#'
-#' Calls (objects of type `language`) do not necessarily call a named
-#' function. They can also call an anonymous function or the result of
-#' some other expression. The language subtypes are organised around
-#' the kind of object being called:
-#'
-#' * For regular calls to named function, `switch_lang()` returns
-#'   "named".
-#'
-#' * Sometimes the function being called is the result of another
-#'   function call, e.g. `foo()()`, or the result of another
-#'   subsetting call, e.g. `foo$bar()` or `foo@bar()`. In this case,
-#'   the call head is not a symbol, it is another call (e.g. to the
-#'   infix functions `$` or `@`). The call subtype is said to be
-#'   "recursive".
-#'
-#' * A special subset of recursive calls are namespaced calls like
-#'   `foo::bar()`. `switch_lang()` returns "namespaced" for these
-#'   calls. It is generally a good idea if your function treats
-#'   `bar()` and `foo::bar()` similarly.
-#'
-#' * Finally, it is possible to have a literal (see [is_expression()] for a
-#'   definition of literals) as call head. In most cases, this will be
-#'   a function inlined in the call (this is sometimes an expedient
-#'   way of dealing with scoping issues). For calls with a literal
-#'   node head, `switch_lang()` returns "inlined". Note that if a call
-#'   head contains a literal that is not function, something went
-#'   wrong and using that object will probably make R crash.
-#'   `switch_lang()` issues an error in this case.
-#'
-#' The reason we use the term _node head_ is because calls are
-#' structured as tree objects. This makes sense because the best
-#' representation for language code is a tree whose hierarchy is
-#' determined by the order of operations. See [node] for more on this.
-#'
-#'
-#' @section Life cycle:
-#'
-#' All these functions are in the questioning stage and likely to be
-#' removed from the package.
-#'
-#' @inheritParams switch_type
-#' @param .x,x A language object (a call). If a formula quote, the RHS
-#'   is extracted first.
-#' @param ... Named clauses. The names should be types as returned by
-#'   `lang_type_of()`.
-#'
-#' @keywords internal
-#' @export
-#' @examples
-#' # Named calls:
-#' lang_type_of(~foo())
-#'
-#' # Recursive calls:
-#' lang_type_of(~foo$bar())
-#' lang_type_of(~foo()())
-#'
-#' # Namespaced calls:
-#' lang_type_of(~base::list())
-#'
-#' # For an inlined call, let's inline a function in the head node:
-#' call <- quote(foo(letters))
-#' call[[1]] <- base::toupper
-#'
-#' call
-#' lang_type_of(call)
-switch_lang <- function(.x, ...) {
-  switch(lang_type_of(.x), ...)
-}
-#' @rdname switch_lang
-#' @export
-coerce_lang <- function(.x, .to, ...) {
-  msg <- paste0("Can't convert ", type_of(.x), " to ", .to, "")
-  switch(lang_type_of(.x), ..., abort(msg))
-}
-#' @rdname switch_lang
-#' @export
-lang_type_of <- function(x) {
-  x <- get_expr(x)
-  stopifnot(typeof(x) == "language")
-
-  type <- typeof(node_car(x))
-  if (type == "symbol") {
-    "named"
-  } else if (is_namespaced_symbol(node_car(x))) {
-    "namespaced"
-  } else if (type == "language") {
-    "recursive"
-  } else if (type %in% c("closure", "builtin", "special")) {
-    "inlined"
-  } else {
-    abort("corrupt language object")
-  }
-}
-
-# Exported `switch_lang()` function is likely to be deprecated in the
-# future. Use switch_call() internally:
-switch_call <- switch_lang
-
 #' Is an object copyable?
 #'
 #' When an object is modified, R generally copies it (sometimes
@@ -759,13 +465,14 @@ switch_call <- switch_lang
 #' structure(base::list, foo = "bar")
 #' str(base::list)
 is_copyable <- function(x) {
-  switch_type(x,
+  switch(typeof(x),
     NULL = ,
     char = ,
     symbol = ,
-    primitive = ,
+    special = ,
+    builtin = ,
     environment = ,
-    pointer =
+    externalptr =
       FALSE,
     TRUE
   )
